@@ -1,4 +1,4 @@
-import os
+import os, sys
 # for output which reports a local time
 os.environ['TZ'] = 'GMT'
 import shutil
@@ -17,21 +17,34 @@ class MagicTest(unittest.TestCase):
             except TypeError:
                 filename = os.path.join(self.TESTDATA_DIR.encode('utf-8'), filename)
 
-            with open(filename, 'rb') as f:
-                value = m.from_buffer(f.read())
-                expected_value_bytes = expected_value.encode('utf-8')
-                self.assertEqual(value, expected_value_bytes)
+            
+            if type(expected_value) is not tuple:
+                expected_value = (expected_value,)
 
-            value = m.from_file(filename)
-            self.assertEqual(value, expected_value_bytes)
-        
+            for i in expected_value:
+                with open(filename, 'rb') as f:
+                    buf_value = m.from_buffer(f.read())
+
+                file_value = m.from_file(filename)
+                if buf_value == i and file_value == i:
+                    break
+            else:
+                self.assertTrue(False, "no match for " + repr(expected_value))
+
+    def test_from_buffer_str_and_bytes(self):
+        m = magic.Magic(mime=True)
+        s = '#!/usr/bin/env python\nprint("foo")'
+        self.assertEqual("text/x-python", m.from_buffer(s))
+        b = b'#!/usr/bin/env python\nprint("foo")'
+        self.assertEqual("text/x-python", m.from_buffer(b))
+                
     def test_mime_types(self):
         dest = os.path.join(MagicTest.TESTDATA_DIR, b'\xce\xbb'.decode('utf-8'))
         shutil.copyfile(os.path.join(MagicTest.TESTDATA_DIR, 'lambda'), dest)
         try:
             m = magic.Magic(mime=True)
             self.assert_values(m, {
-                'magic.pyc': 'application/octet-stream',
+                'magic._pyc_': 'application/octet-stream',
                 'test.pdf': 'application/pdf',
                 'test.gz': 'application/gzip',
                 'text.txt': 'text/plain',
@@ -46,10 +59,11 @@ class MagicTest(unittest.TestCase):
         os.environ['TZ'] = 'UTC'  # To get the last modified date of test.gz in UTC
         try:
             self.assert_values(m, {
-                'magic.pyc': 'python 2.4 byte-compiled',
+                'magic._pyc_': 'python 2.4 byte-compiled',
                 'test.pdf': 'PDF document, version 1.2',
                 'test.gz':
-                'gzip compressed data, was "test", last modified: Sun Jun 29 01:32:52 2008, from Unix',
+                ('gzip compressed data, was "test", from Unix, last modified: Sun Jun 29 01:32:52 2008',
+                 'gzip compressed data, was "test", last modified: Sun Jun 29 01:32:52 2008, from Unix'),
                 'text.txt': 'ASCII text',
             })
         finally:
@@ -77,11 +91,21 @@ class MagicTest(unittest.TestCase):
         filename = os.path.join(self.TESTDATA_DIR, 'keep-going.jpg')
 
         m = magic.Magic(mime=True)
-        self.assertEqual(m.from_file(filename), 
-                         'image/jpeg'.encode('utf-8'))
+        self.assertEqual(m.from_file(filename), 'image/jpeg')
         
         m = magic.Magic(mime=True, keep_going=True)
-        self.assertEqual(m.from_file(filename), 'image/jpeg'.encode('utf-8'))
+        self.assertEqual(m.from_file(filename), 'image/jpeg')
 
+
+    def test_rethrow(self):
+        old = magic.magic_buffer
+        try:
+            def t(x,y):
+                raise magic.MagicException("passthrough")
+            magic.magic_buffer = t
+            
+            self.assertRaises(magic.MagicException, magic.from_buffer, "hello", True)
+        finally:
+            magic.magic_buffer = old
 if __name__ == '__main__':
     unittest.main()
